import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

import java.util.List;
import java.util.ArrayList;

/**
 * Wombat. A Wombat moves forward until it can't do so anymore, at
 * which point it turns left. If a wombat finds a leaf, it eats it.
 * 
 * @author Michael Kolling
 * @version 1.0.1
 */
public class Wombat extends Actor
{
    private static final int EAST = 0;
    private static final int WEST = 1;
    private static final int NORTH = 2;
    private static final int SOUTH = 3;

    private int direction;
    private int leavesEaten;

    public Wombat()
    {
        setDirection(EAST);
        leavesEaten = 0;
    }

    /**
     * Do whatever the wombat likes to do just now.
     */
    public void act()
    {
        if(foundLeaf()) {
            eatLeaf();
        }
        else if(facingRock()) {
            turnRandom();
        }
        else if(facingEdge()) {
            turnRandom();
        }
        else {
            move();
        }
    }

    /**
     * Check whether there is a leaf in the same cell as we are.
     */
    public boolean foundLeaf()
    {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        if(leaf != null) {
            return true;
        }
        else {
            return false;
        }
    }
    
    /**
     * Eat a leaf.
     */
    public void eatLeaf()
    {
        Actor leaf = getOneObjectAtOffset(0, 0, Leaf.class);
        if(leaf != null) {
            // eat the leaf...
            getWorld().removeObject(leaf);
            leavesEaten = leavesEaten + 1; 
        }
    }
    
    /**
     * Move one cell forward in the current direction.
     */
    public void move()
    {
        // Don't move if facing a rock or an edge.
        if (facingRock() || facingEdge()) {
            return;
        }
               
        switch(direction) {
            case SOUTH :
                setLocation(getX(), getY() + 1);
                break;
            case EAST :
                setLocation(getX() + 1, getY());
                break;
            case NORTH :
                setLocation(getX(), getY() - 1);
                break;
            case WEST :
                setLocation(getX() - 1, getY());
                break;
        }
    }

    /**
     * Test if the Wombat is facing an edge. 
     * Return true if yes, false otherwise.
     */
    public boolean facingEdge()
    {
        World myWorld = getWorld();
        int x = getX();
        int y = getY();
        
        switch(direction) {
            case SOUTH :
                y++;
                break;
            case EAST :
                x++;
                break;
            case NORTH :
                y--;
                break;
            case WEST :
                x--;
                break;
        }
        
       // test for outside border
        if (x >= myWorld.getWidth() || y >= myWorld.getHeight()) {
            return true;
        }
        else if (x < 0 || y < 0) {
            return true;
        }
        return false;
    }

    /**
     * Test if there is a rock in the cell the Wombat
     * is facing. Return true if yes, false otherwise.
     */
    public boolean facingRock()
    {
        World myWorld = getWorld();
        int x = getX();
        int y = getY();
        
        switch(direction) {
            case SOUTH :
                y++;
                break;
            case EAST :
                x++;
                break;
            case NORTH :
                y--;
                break;
            case WEST :
                x--;
                break;
        }
        
        // test for rock
        List rocks = getWorld().getObjectsAt(x , y, Rock.class);
        
        /* Look up the List class (java.util.List) in the Java API
         * and determine what method to use with the 'rocks' List
         * to determine if there was a rock. Put the correct test
         * in the 'if()' statement below.
         */
        {
        
            Actor Rock = getOneObjectAtOffset(0, 0, Rock.class);
            
            return true;
        

       }
    
   }

    /**
     * Turns towards the left.
     */
    public void turnLeft()
    {
       switch(direction) {
            case SOUTH :
                setDirection(EAST);
                break;
            case EAST :
                setDirection(NORTH);
                break;
            case NORTH :
                setDirection(WEST);
                break;
            case WEST :
                setDirection(SOUTH);
                break;
        }
    }

    /**
     * Turns random.
     */
    public void turnRandom()
    {
        /* Use the Greenfoot random number method to generate a 
         * random direction to face the Wombat. 0..3 are the 
         * possible directions. Put the correct code in the 
         * statement below.
         */
        move(1);
        if (Greenfoot.getRandomNumber(100) < 10)
        {
            turn(Greenfoot.getRandomNumber(90));
            }
       
    }

    /**
     * Sets the direction we're facing.
     */
    public void setDirection(int direction)
    {
        this.direction = direction;
        setImage("wombat-left.gif");
      
        /* In the code below, where could you put the statement
         * to change the Wombat image so that the Wombat faces
         * left correctly. Look in the Actor class to see how to
         * change the image. There is an appropriate image in the
         * scenario images folder ready to use.
         */
        
        switch(direction) {
            case SOUTH :
                setRotation(90);
                break;
            case EAST :
                setRotation(0);
                break;
            case NORTH :
                setRotation(270);
                break;
            case WEST :
                setRotation(180);
                break;
                
            default :
                break;
        }
    }

    /**
     * Tell how many leaves we have eaten.
     */
    public int getLeavesEaten()
    {
        return leavesEaten;
    }
}